@@ -22,6 +22,8 @@
 #ifndef MOD_AUTH_MELLON_H
 #define MOD_AUTH_MELLON_H
 
+#include <stdbool.h>
+
 #include <lasso/lasso.h>
 #include <lasso/xml/saml-2.0/samlp2_authn_request.h>
 #include <lasso/xml/saml-2.0/samlp2_logout_request.h>
@@ -71,13 +73,10 @@
 /* Size definitions for the session cache.
  */
 #define AM_CACHE_KEYSIZE 120
-#define AM_CACHE_VARSIZE 128
-#define AM_CACHE_VALSIZE 512-AM_CACHE_VARSIZE
 #define AM_CACHE_ENVSIZE 128
 #define AM_CACHE_USERSIZE 512
-#define AM_CACHE_MAX_LASSO_IDENTITY_SIZE 1024
-#define AM_CACHE_MAX_LASSO_SESSION_SIZE 32768
-#define AM_CACHE_MAX_LASSO_SAML_RESPONSE_SIZE 65536
+#define AM_CACHE_DEFAULT_ENTRY_SIZE 196608
+#define AM_CACHE_MIN_ENTRY_SIZE 65536
 
 
 /* This is the length of the id we use (for session IDs and
@@ -101,12 +100,15 @@ typedef struct am_mod_cfg_rec {
     int post_count;
     apr_size_t post_size;
 
+    int entry_size;
+
     /* These variables can't be allowed to change after the session store
      * has been initialized. Therefore we copy them before initializing
      * the session store.
      */
     int init_cache_size;
     const char *init_lock_file;
+    apr_size_t init_entry_size;
 
     apr_shm_t *cache;
     apr_global_mutex_t *lock;
@@ -210,6 +212,9 @@ typedef struct am_dir_cfg_rec {
     /* No cookie error page. */
     const char *no_cookie_error_page;
 
+    /* Authorization error page. */
+    const char *no_success_error_page;
+
     /* Login path for IdP initiated logins */
     const char *login_path;
 
@@ -237,10 +242,13 @@ typedef struct am_dir_cfg_rec {
     LassoServer *server;
 } am_dir_cfg_rec;
 
+typedef struct am_cache_storage_t {
+    apr_uintptr_t ptr;
+} am_cache_storage_t;
 
 typedef struct am_cache_env_t {
-    char varname[AM_CACHE_VARSIZE];
-    char value[AM_CACHE_VALSIZE];
+    am_cache_storage_t varname;
+    am_cache_storage_t value;
 } am_cache_env_t;
 
 typedef struct am_cache_entry_t {
@@ -249,16 +257,20 @@ typedef struct am_cache_entry_t {
     apr_time_t expires;
     int logged_in;
     unsigned short size;
-    char user[AM_CACHE_USERSIZE];
+    am_cache_storage_t user;
 
     /* Variables used to store lasso state between login requests
      *and logout requests.
      */
-    char lasso_identity[AM_CACHE_MAX_LASSO_IDENTITY_SIZE];
-    char lasso_session[AM_CACHE_MAX_LASSO_SESSION_SIZE];
-    char lasso_saml_response[AM_CACHE_MAX_LASSO_SAML_RESPONSE_SIZE];
+    am_cache_storage_t lasso_identity;
+    am_cache_storage_t lasso_session;
+    am_cache_storage_t lasso_saml_response;
 
     am_cache_env_t env[AM_CACHE_ENVSIZE];
+
+    apr_size_t pool_size;
+    apr_size_t pool_used;
+    char pool[];
 } am_cache_entry_t;
 
 typedef enum { 
@@ -276,6 +288,13 @@ typedef struct am_envattr_conf_t {
 
 extern const command_rec auth_mellon_commands[];
 
+typedef struct am_error_map_t {
+    int lasso_error;
+    int http_error;
+} am_error_map_t;
+
+extern const am_error_map_t auth_mellon_errormap[];
+
 /* When using a value from a directory configuration structure, a special value is used
  * to state "inherit" from parent, when reading a value and the value is still inherit from, it
  * means that no value has ever been set for this directive, in this case, we use the default
@@ -310,8 +329,11 @@ void am_cookie_set(request_rec *r, const char *id);
 void am_cookie_delete(request_rec *r);
 
 
+void am_cache_init(am_mod_cfg_rec *mod_cfg);
 am_cache_entry_t *am_cache_lock(server_rec *s, 
                                 am_cache_key_t type, const char *key);
+const char *am_cache_entry_get_string(am_cache_entry_t *e,
+                                      am_cache_storage_t *slot);
 am_cache_entry_t *am_cache_new(server_rec *s, const char *key);
 void am_cache_unlock(server_rec *s, am_cache_entry_t *entry);
 
@@ -342,7 +364,7 @@ void am_delete_request_session(request_rec *r, am_cache_entry_t *session);
 
 char *am_reconstruct_url(request_rec *r);
 int am_check_permissions(request_rec *r, am_cache_entry_t *session);
-void am_set_nocache(request_rec *r);
+void am_set_cache_control_headers(request_rec *r);
 int am_read_post_data(request_rec *r, char **data, apr_size_t *length);
 char *am_extract_query_parameter(apr_pool_t *pool,
                                  const char *query_string,
