@@ -19,8 +19,6 @@
  *
  */
 
-#include <stdbool.h>
-
 #include "auth_mellon.h"
 
 /* This is the default endpoint path. Remember to update the description of
@@ -862,6 +860,15 @@ const command_rec auth_mellon_commands[] = {
         " restart the server before any changes to this directive will"
         " take effect. The default value is 100."
         ),
+    AP_INIT_TAKE1(
+        "MellonCacheEntrySize",
+        am_set_module_config_int_slot,
+        (void *)APR_OFFSETOF(am_mod_cfg_rec, entry_size),
+        RSRC_CONF,
+        "The maximum size for a single session entry. You must"
+        " restart the server before any changes to this directive will"
+        " take effect. The default value is 192KiB."
+        ),
     AP_INIT_TAKE1(
         "MellonLockFile",
         am_set_module_config_file_slot,
@@ -1045,6 +1052,15 @@ const command_rec auth_mellon_commands[] = {
         " return a 400 Bad Request error if this is unset and the user"
         " ha disabled cookies."
         ),
+    AP_INIT_TAKE1(
+        "MellonNoSuccessErrorPage",
+        ap_set_string_slot,
+        (void *)APR_OFFSETOF(am_dir_cfg_rec, no_success_error_page),
+        OR_AUTHCFG,
+        "Web page to display if the idp posts with a failed"
+        " authentication error. We will return a 401 Unauthorized error"
+        " if this is unset and the idp posts such assertion."
+        ),
     AP_INIT_TAKE1(
         "MellonSPMetadataFile",
         am_set_filestring_slot,
@@ -1190,7 +1206,7 @@ const command_rec auth_mellon_commands[] = {
     AP_INIT_TAKE1(
         "MellonDoNotVerifyLogoutSignature",
         am_set_do_not_verify_logout_signature,
-        NULL,
+        (void *)APR_OFFSETOF(am_dir_cfg_rec, do_not_verify_logout_signature),
         OR_AUTHCFG,
         "A list of entity of IdP whose logout requests signatures will not "
         "be valided"
@@ -1205,6 +1221,13 @@ const command_rec auth_mellon_commands[] = {
     {NULL}
 };
 
+const am_error_map_t auth_mellon_errormap[] = {
+    { LASSO_PROFILE_ERROR_STATUS_NOT_SUCCESS, HTTP_UNAUTHORIZED },
+#ifdef LASSO_PROFILE_ERROR_REQUEST_DENIED
+    { LASSO_PROFILE_ERROR_REQUEST_DENIED, HTTP_UNAUTHORIZED },
+#endif
+    { 0, 0 }
+};
 
 /* Release a lasso_server object associated with this configuration.
  *
@@ -1264,6 +1287,7 @@ void *auth_mellon_dir_config(apr_pool_t *p, char *d)
     dir->session_length = -1; /* -1 means use default. */
 
     dir->no_cookie_error_page = NULL;
+    dir->no_success_error_page = NULL;
 
     dir->sp_metadata_file = NULL;
     dir->sp_private_key_file = NULL;
@@ -1418,6 +1442,10 @@ void *auth_mellon_dir_merge(apr_pool_t *p, void *base, void *add)
                                      add_cfg->no_cookie_error_page :
                                      base_cfg->no_cookie_error_page);
 
+    new_cfg->no_success_error_page = (add_cfg->no_success_error_page != NULL ?
+                                     add_cfg->no_success_error_page :
+                                     base_cfg->no_success_error_page);
+
 
     new_cfg->sp_metadata_file = (add_cfg->sp_metadata_file ?
                                  add_cfg->sp_metadata_file :
@@ -1550,8 +1578,11 @@ void *auth_mellon_server_config(apr_pool_t *p, server_rec *s)
     mod->post_count = post_count;
     mod->post_size  = post_size;
 
+    mod->entry_size = AM_CACHE_DEFAULT_ENTRY_SIZE;
+
     mod->init_cache_size = 0;
     mod->init_lock_file = NULL;
+    mod->init_entry_size = 0;
 
     mod->cache      = NULL;
     mod->lock       = NULL;
